/**
 * @file TP_RS232.c
 * @brief Serial communication programm example for STM32F107VC on MCBSTM32C board.
 * @author P.Douay
 * @version 0.1
 * @date january, 19th 2011 Creation
 * @date 07/10/2020 P.Douay : 
 *				modified for dual versions of displays 
 *				change project settings to use Segger debugger
 * @see RM0008 manual
 *                            
 * main programm file.
 *
 */
 

#include <stdio.h>                          /* standard I/O .h-file           */
#include <ctype.h>                          /* character functions            */
#include <string.h>                         /* string functions            */

#include <stm32f10x_cl.h>
#include "stm32f10x_bxcan.h"

#include "measure.h"                        /* global project definition file */
#include "GLCD.h"                                 // GLCD function prototypes

#define Qresol	(float)(3.3/4096)

#define FONT_6_8H			0                    /* premire fonte   */
#define FONT_16_24H		1                    /* seconde fonte   */
#define FONT					FONT_16_24H

typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

unsigned long TimeTick;                     /* counts 10ms timeTicks          */


struct interval setinterval;                /* interval setting values        */
struct interval interval;                   /* interval counter               */

volatile int measurement_interval = 0;      /* measurement interval over      */
volatile int mdisplay = 0;                  /* measurement display requested  */
volatile int startflag = 0;                 /* start measurement recording    */

struct mrec current;                        /* current measurements           */

#define SCNT 20                             /* number of records              */

struct mrec save_record[SCNT];              /* buffer for measurements        */
int sindex;                                 /* save index                     */
int savefirst;                              /* save first index               */
CanRxMsg Rx0Message,Rx1Message;				// messages de rception CAN	*/
u8 ret=0;
u8 req=0;
u8 adComplete=0;							// adc conversion complete
u8 canTxStarted = 0;						// Can message initiated
u8 canTxOk = 1;								// Can transmission is OK*/
u8 canTxMb;							// mail box used for transmission
u16 convResult=0xA55A;						//AD conversion value	
float adv = 1.1;							// AD voltage	

volatile u8 canRes;
char message[11];   
volatile u16 adBCD,tmp;
int ret = 0;

char ERROR_STR [] = "\n*** ERROR: %s\n";    /* ERROR message string in code   */



#define WRONGINDEX 0xffff                   /* error signal for wrong index   */


/******************************************************************************/
/*               Initialises the LEDs                         */
/******************************************************************************/
void io_Init (void) {

  RCC->APB2ENR |= 0x00000060;

  RCC->APB2ENR |= (1<<4);                      /* enable clock for GPIOD      */
  GPIOD->CRH &= 0x00000FFF;                    /* Configure GPIO  */
  GPIOD->CRH |= 0x44444000;

  RCC->APB2ENR |= (1<<6);                      /* enable clock for GPIOE      */
  GPIOE->CRH  = 0x33333333;                    /* Configure GPIO for LEDs     */

}

/******************************************************************************/
/*               Initialises the TIM3 for 100ms @ 36MHz                         */
/******************************************************************************/
#define TIM3_FULL_REMAP_MASK 0x00000C00		
#define T3_CH3_EN	((short)0x0100)

void tim3_Init (void) {

  RCC->APB1ENR |= (1<<1);                      /* enable clock for TIM3       */
  RCC->APB2ENR |=  1  ;    	/* Enable AFIO clock (mandatory for the following instructions)*/
  AFIO->MAPR |= TIM3_FULL_REMAP_MASK; 		/* remap Timer3 to port C (AFIO must be enabled)*/ 

  RCC->APB2ENR |= 0x00000010;
  GPIOC->CRH  &= 0xFFFFFFF0;  /* erase PC8=T3_CH3 conf */
  GPIOC->CRH  |= 0x0000000B;  /* Configure PC8 as alt. output with 2MHz limit -> A */

  TIM3->PSC = 36000-1;                         /* set prescaler               */
  TIM3->ARR = 200-1;					       /* set auto-reload             */
  TIM3->CR1 = 0;                               /* reset command register 1    */
  TIM3->CR2 = 0;                               /* reset command register 2    */
 
  // pour contrle de priodicit avec l'oscilloscope
  TIM3->CCER &=~T3_CH3_EN; 	// disable channel 3
   
  TIM3->CCMR2 &= 0xFF00;
  TIM3->CCMR2 |= 0x0030;	// bascule  chaque vnement 

  TIM3->CCER |= T3_CH3_EN; 	// enable channel 3
  TIM3->CCR3 = 0;
  
  TIM3->SR = 0;
  
  TIM3->DIER = (1<<0);                         /* Update interrupt enabled    */
  NVIC_EnableIRQ(TIM3_IRQn);                   /* enable TIM3 interrupt       */
  TIM3->CR1 |= (1<<0);                         /* Enable Timer                */

}

/******************************************************************************/
/*               Initialises the Analog/Digital converter                     */
/*               PC4 (ADC Channel14) is used as analog input                  */
/******************************************************************************/
void adc_Init (void) {

  RCC->APB2ENR |= (1<<4);                      /* enable clock for GPIOC      */
  GPIOC->CRL &= 0xFFF0FFFF;                    /* Configure ADC.14 input.     */

  RCC->APB2ENR |= (1<<9);                      /* enable clock for ADC1       */
  ADC1->SQR1  = 0x00000000;                    /* one conversion              */
  ADC1->SQR3  = (14<< 0);                      /* set order to chn14          */
  ADC1->SMPR1 = ( 5<<12);                      /* sample time (55,5 cycles)   */ 

  ADC1->CR1   =  0x00000900;                   /* independant mode, SCAN mode plus disc. mode*/
  ADC1->CR2   =  0x000E0001;                   /* data align right            */
                                               /* discontinuous conversion       */
                                               /* EXTSEL = SWSTART            */
                                               /* enable ADC                  */ 
  ADC1->CR2  |= 0x00000008;                    /* Reset calibration           */
  while (ADC1->CR2 & 0x00000008);

  ADC1->CR2  |= 0x00000004;                    /* Start calibration           */
  while (ADC1->CR2 & 0x00000004);

  ADC1->CR2  |=  0x00500000;                   /* start SW conversion         */

}
/******************************************************************************/
/*                CAN1 RX0 interrupt service function                  */
/******************************************************************************/
void CAN1_RX0_IRQHandler(void)
{
  Rx0Message.StdId=0x62;
  Rx0Message.ExtId=0x62;
  Rx0Message.IDE=0;
  Rx0Message.DLC=2;
  Rx0Message.FMI=0;
  Rx0Message.Data[0]=0x63;
  Rx0Message.Data[1]=0x64;

	u8 FifoNumber = 0;
	CAN_Receive(FifoNumber, &Rx0Message);
 // bxCAN receive code to be implented here	for FIFO0
}

/******************************************************************************/
/*                CAN1 RX1 interrupt service function                  */
/******************************************************************************/
void CAN1_RX1_IRQHandler(void)
{
  Rx1Message.StdId=0x00;
  Rx1Message.ExtId=0x00;
  Rx1Message.IDE=0;
  Rx1Message.DLC=0;
  Rx1Message.FMI=0;
  Rx1Message.Data[0]=0x00;
  Rx1Message.Data[1]=0x00;

// bxCAN receive code to be implented here for FIFO1

}



u8 CAN_TX_Message(void)
{

  CanTxMsg TxMessage;
  u8 TransmitMailbox;

   /* transmit */
 TxMessage.StdId = 0x50;
 TxMessage.RTR = 0x0;
 TxMessage.IDE = 0x0;
 TxMessage.DLC = 0x1;
 TxMessage.Data[0]= 0x10;
 TransmitMailbox = CAN_Transmit(&TxMessage);

  return  TransmitMailbox;
}

u8 CAN_TX_mv(void)
{

  CanTxMsg TxMessage;
  u8 TransmitMailbox;

  return  TransmitMailbox;
}

u8 CAN_TX_Poll(u8 txMailBox)
{
  u32 i = 0;
    i = 0;
  while((CAN_TransmitStatus(txMailBox) != CANTXOK) && (i != 0xFF))
  {
    i++;
  }
  if (i>=0xFF) return 0;
  else 	 return 1;
}

  /******************************************************************************/
/*                Timer Counter 3 interrupt service function                  */
/*                executes each 1ms @ 25 MHz Crystal Clock                     */
/******************************************************************************/
void TIM3_IRQHandler (void) {

  if (TIM3->SR & (1<<0)) {                     /* UIF interrupt ?             */
    
	if (ADC1->SR & 0x2)						// une mesure disponible 
	{
		adComplete = 1;
		convResult += 1;	
		ADC1->CR2 |= (1<<22);			//relance une conversion
		if (canTxOk)
		{	
			canTxMb = CAN_TX_Message(); // canTxStarted set to 1 
		}
	}
    TIM3->SR &= ~(1<<0);                       /* clear UIF flag              */
  }

}

/**
 * @fn   void main(void)
 * @return int 
 * @brief main program
 * @author P.Douay
 * @version 0.1
 * @date january, 19th 2011 Creation
 * @date XX/XX/XXXX Y.YYYYY : details of changes

 * @ see 
 *
 * Main clocks and peripherals initialisations before main loop treatments
 */

#define NB_AD	4
int main (void)  {                             
  u16 adTab[NB_AD],filteredVal,speedVal;//,Speed;
  char dispBuf [20+1];            /* display buffer        */
  int i=0;                                       
  const char TP[]="   TP bus CAN V2    ";
 
  /*--------------------------------------------------------------------------*/
  SystemInit();                                /* Initialize clocks           */
  io_Init();                                   /* Initialize LEDs & Joystick  */
  
    /* CAN register init */
  CAN_DeInit();     
  canRes = CAN_Init();	
  CAN_FilterInit();
 
  GLCD_Init();                                 /* Initialize the GLCD         */
	CAN1->IER |= CAN_IT_FMP0;
	NVIC_EnableIRQ(CAN1_RX0_IRQn);
  GLCD_Clear(White);                           /* Clear the GLCD              */

  GLCD_SetBackColor(Blue);                     /* Set the Back Color          */
  GLCD_SetTextColor(White);                    /* Set the Text Color          */
  GLCD_DisplayString(0, 0, FONT , "   MCBSTM32C        ");
  GLCD_DisplayString(1, 0, FONT , (unsigned char*)TP);
  GLCD_DisplayString(2, 0, FONT , " www.esiee-amiens.fr");
  GLCD_SetBackColor(White);                    /* Set the Back Color          */
  GLCD_SetTextColor(Blue);                     /* Set the Text Color          */
  message[0] = canRes +'0';
  message[1] = '\0';
  sprintf(dispBuf,"INIT : ");
  strcat(dispBuf,message); 
  GLCD_DisplayString(4, 0, FONT , (unsigned char*)dispBuf);
  
  adc_Init();                                  /* Initialise A/D converter    */
  ADC1->CR2 |= (1<<22);			//Start conversions
  tim3_Init(); 								   /* Initialize Timer #3         */
  
  // valider l'it sur fifo0 pending (partie 3)
          
         
  for (i=0;i<NB_AD-1;i++) adTab[i]=0; // init de la table de filtrage
 
  while (1)  {                                 /* loop forever                */
  
  
	if (req==1)
	{
		req=0;
		adBCD = 3456;
		sprintf(dispBuf,"Rq : %4i",adBCD); 
		GLCD_DisplayString(8, 9, FONT , (unsigned char*)dispBuf);
	}

	if(canTxStarted)
	{
		canRes = CAN_TX_Poll(canTxMb); // check of the sending status
		canTxStarted=0;	 
		message[0] = canRes +'0';
  		message[1] = '\0';
		sprintf(dispBuf," TX : ");
  		strcat(dispBuf,message); 
  		GLCD_DisplayString(4, 10, FONT , (unsigned char*)dispBuf);
	}	

	if (adComplete)
	{
		adComplete = 0 ;
		adv = (float)convResult;
		sprintf(dispBuf,"A/D : %.2f",adv); 
		GLCD_DisplayString(6, 0, FONT , (unsigned char*)dispBuf);
	}
	
	
	for (i=0;i<5000;i++);
	GLCD_DisplayString(4, 10, FONT , "           ");
   }      
}

